@@ -23,7 +23,9 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-`convert` <- function(from, to = NULL, declared = TRUE, embed = TRUE, ...) {
+`convert` <- function(
+    from, to = NULL, declared = TRUE, recode = TRUE, embed = TRUE, ...
+) {
     if (missing(from)) {
         admisc::stopError("Argument 'from' is missing.")
     }
@@ -34,6 +36,30 @@
 
     dots <- list(...)
 
+    user_na <- TRUE # force reading the value labels
+    if (
+        is.element("user_na", names(dots)) && is.atomic(dots$user_na) && 
+        length(dots$user_na) == 1 && is.logical(dots$user_na)
+    ) {
+        user_na <- dots$user_na
+    }
+
+    encoding <- "latin1"
+    if (
+        is.element("encoding", names(dots)) && is.atomic(dots$encoding) && 
+        length(dots$encoding) == 1 && is.character(dots$encoding)
+    ) {
+        encoding <- dots$encoding
+    }
+
+    chartonum <- TRUE
+    if (
+        is.element("chartonum", names(dots)) && is.atomic(dots$chartonum) && 
+        length(dots$chartonum) == 1 && is.logical(dots$chartonum)
+    ) {
+        chartonum <- dots$chartonum
+    }
+
     targetOS <- "" # for the XML file output of function exportDDI()
     if (is.element("OS", names(dots))) {
         targetOS <- dots$OS
@@ -202,23 +228,60 @@
             # }
         }
         else if (tp_from$fileext == "SAV") {
-            data <- haven::read_spss(from, user_na = TRUE)
+            fargs <- names(formals(read_sav))
+            arglist <- dots[is.element(names(dots), fargs)]
+            arglist$file <- from
+            arglist$user_na <- user_na
+            arglist$encoding <- encoding
+            data <- do.call(haven::read_sav, arglist)
         }
         else if (tp_from$fileext == "POR") {
-            data <- haven::read_por(from, user_na = TRUE)
+            fargs <- names(formals(read_por))
+            arglist <- dots[is.element(names(dots), fargs)]
+            arglist$file <- from
+            arglist$user_na <- user_na
+            data <- do.call(haven::read_por, arglist)
         }
         else if (tp_from$fileext == "DTA") {
-            data <- haven::read_dta(from)
-            data <- recodeMissing(
-                data,
-                to = "SPSS",
-                dictionary = dictionary,
-                to_declared = FALSE,
-                error_null = FALSE
-            )
+            fargs <- names(formals(read_dta))
+            arglist <- dots[is.element(names(dots), fargs)]
+            arglist$file <- from
+            arglist$encoding <- encoding
+            data <- do.call(haven::read_dta, arglist)
+
+            # return(list(data = data, to = to, dictionary = dictionary, chartonum = chartonum, to_declared = FALSE, error_null = FALSE))
+            if (recode) {
+                data <- recodeValues(
+                    dataset = data,
+                    to = "SPSS",
+                    dictionary = dictionary,
+                    chartonum = chartonum,
+                    to_declared = FALSE,
+                    error_null = FALSE
+                )
+            }
+            else {
+                declared <- FALSE
+            }
         }
         else if (tp_from$fileext == "SAS7BDAT") {
-            data <- haven::read_sas(from)
+            fargs <- names(formals(read_sas))
+            arglist <- dots[is.element(names(dots), fargs)]
+            arglist$file <- from
+            arglist$encoding <- encoding
+            data <- do.call(haven::read_sas, arglist)
+
+            if (recode) {
+                data <- recodeValues(
+                    dataset = data,
+                    to = "SPSS",
+                    dictionary = dictionary,
+                    chartonum = chartonum,
+                    to_declared = FALSE,
+                    error_null = FALSE
+                )
+            }
+
         }
         else if (tp_from$fileext == "RDS" & !Robject) {
             data <- readr::read_rds(from)
@@ -228,6 +291,7 @@
         }
 
         codeBook <- getMetadata(data, error_null = FALSE)
+        codeBook$fileDscr$fileName <- tp_from$filenames
     }
 
     # The current OS might not always be the same with the target OS aboe
@@ -293,22 +357,24 @@
                         }
                     }
                 }
-
-                callist <- list(
-                    data = recodeMissing(
+                arglist <- list(
+                    data = recodeValues(
                         data,
                         to = "Stata",
                         dictionary = dictionary,
+                        chartonum = chartonum,
+                        to_declared = FALSE,
                         error_null = FALSE
                     )
                 )
             }
             else {
-                callist <- list(data = data)
+                arglist <- list(data = data)
             }
+            # return(data)
 
             if (is.element("version", names(dots))) {
-                callist$version <- dots$version
+                arglist$version <- dots$version
             }
 
             # if (requireNamespace("readstata13", quietly = TRUE)) {
@@ -326,8 +392,10 @@
             #     do.call(readstata13::save.dta13, callist)
             # }
             # else {
-                callist$path <- to
-                do.call(haven::write_dta, callist)
+                arglist$path <- to
+                
+                do.call(haven::write_dta, arglist)
+                # return(invisible(arglist$data))
             # }
         }
         else if (identical(tp_to$fileext, "RDS")) {
