@@ -42,7 +42,7 @@ getFiles <- function(path = ".", type = "*", currdir) {
             paste(
                 "The directory",
                 path,
-                "is empty.\n\n"
+                "is empty."
             )
         )
     }
@@ -55,7 +55,7 @@ getFiles <- function(path = ".", type = "*", currdir) {
             paste(
                 "The directory \"",
                 path,
-                "\" doesn't contain any known files.\n\n",
+                "\" doesn't contain any known files.",
                 sep = ""
             )
         )
@@ -106,7 +106,7 @@ getFiles <- function(path = ".", type = "*", currdir) {
                     type,
                     " type file in the directory \"",
                     path,
-                    "\"\n\n",
+                    "\"",
                     sep = ""
                 )
             )
